#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yumexUtils.py,v 1.27 2006/02/18 06:24:39 timlau Exp $

""" Yum Extender Utility Classes """
 
import sys, os, gobject, glob, shutil, datetime
import gtk
import gtk.glade
import pygtk
import time

from types import *
from yumex.yumexi18n import _

import yumex.yumexDialogs 
from yumex.yumexPackages import yumexPackageList, yumexPackage

# Handle compability with yum 2.4.x
import yum
if yum.__version__ >= '2.5,0':
    from yum.config import *
    from yum.parser import IncludingConfigParser
else:
    from yumex.yum24Xcomp import *

import yumex.yumexConst as const
import ConfigParser
from threading import Thread


class PackageView:
    ''' Abstact class to handle view with packages '''
    
    def __init__(self,treeview):
        self.view = treeview
        self.headers = [_( "Package" ), _( "Ver" ), _( "Summary" ), _( "Repo" ), _( "Architecture" ), _( "Size" )]

    def on_toggled( self, widget, path):
        """ Package selection handler """
        self.curView.on_toggled( widget, path )
        
    def on_recent_column_click( self, col ): 
        self.curView.on_recent_column_click(col)
        self._setRecentHeader()
        
    def _setRecentHeader(self):
        col = self.view.get_column( 1 )
        state = self.curView.RecentState
        if state == 1:
            col.set_title( "+" )
        elif state == 2:
            col.set_title( "V")
        else:
            col.set_title( "" )

    def on_select_column_click( self, col ): 
        self.curView.on_select_column_click(col)
        self._setSelectHeader()

    def _setSelectHeader(self):
        col = self.view.get_column( 0 ) 
        if self.curView.SelectState:
            col.set_title( "X" )
        else:
            col.set_title( "" )
                          
    def setup_view( self):
        """
        Setup Models, TreeView Columns and connect event handlers.
        """
        self.tip = gtk.Tooltips()
        # Setup selection column
        cell1 = gtk.CellRendererToggle()    # Selection
        cell1.set_property( 'activatable', True )
        column1 = gtk.TreeViewColumn( "    ", cell1 )
        column1.add_attribute( cell1, "active", 0 )
        column1.set_resizable( True )
        column1.set_sort_column_id( -1 )            
        self.view.append_column( column1 )
        cell1.connect( "toggled", self.on_toggled )            
        column1.set_clickable( True )
        column1.connect( "clicked", self.on_select_column_click ) 

        # Setup resent column
        cell2 = gtk.CellRendererPixbuf()    # new
        cell2.set_property( 'stock-id', gtk.STOCK_ADD )
        column2 = gtk.TreeViewColumn( "    ", cell2 )
        column2.set_cell_data_func( cell2, self.new_pixbuf )
        column2.set_resizable( True )
        column2.set_sort_column_id( -1 )            
        self.view.append_column( column2 )
        column2.set_clickable( True )
        column2.connect( "clicked", self.on_recent_column_click ) 
        
        for i in [1, 2, 3, 4, 5, 6]:       
            self.create_text_column( self.headers[i-1], self.view, i )
        self.view.set_search_column( 1 )
        self.view.set_reorderable( False )

    def create_text_column( self, hdr, view, colno ):
        """ 
        Create a TreeViewColumn with text and set
        the sorting properties and add it to the view
        """
        cell = gtk.CellRendererText()    # Size Column
        column = gtk.TreeViewColumn( hdr, cell, text=colno )
        column.set_resizable( True )
        if not colno in  [2, 5]:
            column.set_sort_indicator( True )
            column.set_sort_column_id( colno )
            column.set_sort_order( gtk.SORT_ASCENDING ) 
        else:
            column.set_sort_column_id( -1 )
        view.append_column( column )        
        
    def new_pixbuf( self, column, cell, model, iter ):
        """ 
        Cell Data function for recent Column, shows pixmap
        if recent Value is True.
        """
        path = model.get_string_from_iter(iter)
        row = self.curView.get_child_path_row( path )
        pkg = row[0]
        action = pkg.queued
        if action:            
            if action in ('u','i'):
                icon = gtk.STOCK_APPLY
            else:
                icon = gtk.STOCK_CANCEL
            cell.set_property( 'visible', True )
            cell.set_property( 'stock-id', icon )
        else:
            cell.set_property( 'visible', pkg.recent )
            cell.set_property( 'stock-id', gtk.STOCK_NEW )

    def get_selected(self):
        return self.curView.packageList.get_selected()


class Packages(PackageView):
    def __init__(self,ui,treeview,progress):
        PackageView.__init__(self,treeview)
        self.ui = ui
        self.progress = progress
        self.activePage = 'update'
        self.pkgViews = { 'install' : pkgView(self.progress),
                          'update' : pkgView(self.progress),
                          'remove' : pkgView(self.progress)}        
        self.curView = self.pkgViews[self.activePage]
        self.setup_view()
        
    def set_page(self,page):
        self.activePage = page
        self.curView = self.pkgViews[self.activePage]
        self.view.set_model(self.curView.model)
        self.ui.pkgHeader.set_markup(const.PACKAGE_PAGE_HEADERS[page])
        self.ui.pageImage1.set_from_file ( const.PACKAGE_PAGE_IMAGES[page])
        self._setRecentHeader()
        self._setSelectHeader()
        self.view.set_search_column( 1 )
        
        
    def get_packageList(self,page=None):
        if page == None:
            return self.curView.packageList
        else:
            return self.pkgViews[page].packageList
            
    def get_packageViews(self):
        views = []
        for page in const.PACKAGE_PAGE_LIST:
            views.append(self.pkgViews[page])
        return views           
        
    def findVer(self,name,arch,page=None):
        return self.get_packageList(page).find_ver(name,arch)
    
    def findPkg(self,name,ver,arch,page=None):
        return self.get_packageList(page).find_pkg_by_nva(name,ver,arch)
        
        
class pluginView:
    def __init__( self, widget ):
        self.view = widget
        self.model = self.setup_view()
        self.plugins = self.load()
        self.populate_view()
    
    def on_toggled( self, cell, path, model ):
        #print path
        newstate = not model[path][0]
        model[path][0] = newstate
        conf = model[path][2]
        if newstate:
            val = '1'
        else:
            val = '0'
        conf.set( 'main', 'enabled', val )
        return

    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.ListStore( 'gboolean', gobject.TYPE_STRING, gobject.TYPE_PYOBJECT )
        self.view.set_model( model )
        # Setup Selection Column
        cell1 = gtk.CellRendererToggle()    # Selection
        cell1.set_property( 'activatable', True )
        column1 = gtk.TreeViewColumn( "    ", cell1 )
        column1.add_attribute( cell1, "active", 0 )
        column1.set_resizable( True )
        self.view.append_column( column1 )
        cell1.connect( "toggled", self.on_toggled, model )             
        cell2 = gtk.CellRendererText()
        column2= gtk.TreeViewColumn( _( "Plugin" ), cell2, text=1 )
        column2.set_resizable( True )
        self.view.append_column( column2 )
        return model
    
    def populate_view( self ):
        self.model.clear()
        keys = self.plugins.keys()
        for key in keys:
            conf = self.plugins[key]
            state = conf.get( 'main', 'enabled' ) == '1'
            el = [state, key, conf]
            self.model.append( el )
        
    def dump( self ):
        for row in self.model:
            print "--> %s is %s" % ( row[1], row[0] )
            
    def load( self ):
        """ Create a dict with parsers to availible plugings"""
        dict = {}
        for modulefile in glob.glob( '/etc/yum/pluginconf.d/*.conf' ):
            #print "Loading : %s" % modulefile
            dir, modname = os.path.split( modulefile )
            modname = modname.split( '.conf' )[0]
            try:
                parser = IncludingConfigParser()
                parser.read( modulefile )
                dict[modname] = parser
            except:
                print "Error loading : %s" % modulefile
                
        return dict
        
            
    def save( self ):
        """ Write plugins configs back to disk """
        for row in self.model:
            fp = open( "/etc/yum/pluginconf.d/%s.conf" % row[1], "wt" )
            row[2].write( fp )
            fp.close()
            

class queueView:
    """ Queue View Class"""
    def __init__( self, widget):
        self.view = widget
        self.model = self.setup_view()
        self.packages = {}
        self.packages['i'] = []
        self.packages['u'] = []
        self.packages['r'] = []

    def clear( self ):
        del self.packages
        self.packages = {}
        self.packages['i'] = []
        self.packages['u'] = []
        self.packages['r'] = []
        self.populate_view()
        
    def get( self, action = None ):        
        if action == None:
            return self.packages
        else:
            return self.packages[action]
            
        
    def add( self, pkg, action ):
        list = self.packages[action]
        if not pkg in list:
            list.append( pkg )
        self.packages[action] = list

    def remove( self, pkg, action ):
        list = self.packages[action]
        if pkg in list:
            list.remove( pkg )
        self.packages[action] = list

    def on_toggled( self, cell, path, user_data ):
        model, column = user_data
        #print path
        model[path][column] = not model[path][column]
        return

    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.TreeStore( gobject.TYPE_STRING, gobject.TYPE_STRING )           
        self.view.set_model( model )
        cell1 = gtk.CellRendererText()
        column1= gtk.TreeViewColumn( _( "Packages" ), cell1, markup=0 )
        column1.set_resizable( True )
        self.view.append_column( column1 )

        cell2 = gtk.CellRendererText()
        column2= gtk.TreeViewColumn( _( "Summary" ), cell2, text=1 )
        column2.set_resizable( True )
        self.view.append_column( column2 )
        model.set_sort_column_id( 0, gtk.SORT_ASCENDING )
        self.view.get_selection().set_mode( gtk.SELECTION_MULTIPLE )
        return model
    
    def deleteSelected( self ):
        rmvlist = []
        model, paths = self.view.get_selection().get_selected_rows()
        for p in paths:
            row = model[p]
            if row.parent != None:
                rmvlist.append( row[0] )
        for pkg in self.getPkgsFromList(rmvlist):
            pkg.queued = None
        f = lambda x: str( x ) not in rmvlist
        for action in ['u', 'i', 'r']:
            list = self.packages[action]
            if list:
                self.packages[action] = filter( f, list )
        self.populate_view()


    def getPkgsFromList(self,rlist):
        rclist = []
        f = lambda x: str( x ) in rlist
        for action in ['u', 'i', 'r']:
            list = self.packages[action]
            if list:
                rclist += filter( f, list )
        return rclist

    def populate_view( self ):
        """ Populate a Notebook page with data """
        self.model.clear()
        label = _( "<b>Packages To Update</b>" )
        list = self.packages['u']
        if len( list ) > 0:
            self.populate_list( label, list )
        label = _( "<b>Packages To Install</b>" )
        list = self.packages['i']
        if len( list ) > 0:
            self.populate_list( label, list )
        label = _( "<b>Packages To Remove</b>" )
        list = self.packages['r']
        if len( list ) > 0:
            self.populate_list( label, list )
        self.view.expand_all()
            
    def populate_list( self, label, list ):
        parent = self.model.append( None, [label, ""] )
        for pkg in list:
            self.model.append( parent, [str( pkg ), pkg.summaryFirst] )
        

class grpInfoView(PackageView):
    """ Group Info View Class"""
    def __init__( self, widget,progress ):
        PackageView.__init__(self,widget)
        self.progress = progress
        self.curView = pkgView(self.progress)
        self.setup_view()
        self.view.set_model(self.curView.model)
        
    def populate_view( self, data):
        """ Populate a Notebook page with data """
        self.curView.packageList.clean()
        self.curView.packageList.add_list(data)

    def on_toggled( self, widget, path):
        """ Package selection handler """
        pass
        
            

class grpView:
    """ Group View Class"""
    def __init__( self, widget ):
        self.view = widget
        self.model = self.setup_view()
    
    def setup_view( self ):
        """ Create Notebook list for single page  """
        model = gtk.TreeStore( str, str, int )           
        #model = gtk.TreeStore( gobject.TYPE_STRING,)           
        model.set_sort_column_id(0 ,gtk.SORT_ASCENDING)
        self.view.set_model( model )
        
        cell1 = gtk.CellRendererText()
        column1= gtk.TreeViewColumn( _( "Categories / Groups" ), cell1, markup=0 )
        column1.set_resizable( True )
        self.view.append_column( column1 )
                # Setup resent column
        cell2 = gtk.CellRendererPixbuf()    # installed
        cell2.set_property( 'stock-id', gtk.STOCK_ADD )
        column2 = gtk.TreeViewColumn( "    ", cell2 )
        column2.set_cell_data_func( cell2, self.new_pixbuf )
        self.view.append_column( column2 )
        
        cell3 = gtk.CellRendererText()
        column3= gtk.TreeViewColumn( _( "Description" ), cell3, markup=1 )
        column3.set_resizable( True )
        self.view.append_column( column3 )
        
        return model

    def new_pixbuf( self, column, cell, model, iter ):
        state = model.get_value( iter, 2 )
        if state == 1:
            cell.set_property( 'stock-id', gtk.STOCK_FLOPPY )
            cell.set_property( 'visible', True )
        else:
            cell.set_property( 'visible', False )

    def populate_view( self, comps ):
        """ Populate a Notebook page with data """
        self.model.clear()
        cats = comps.categories
        for cat in cats:
            grps = map( lambda x: comps.return_group( x ), 
               filter( lambda x: comps.has_group( x ), cat.groups ) )
            grplist = []
            for grp in grps:
                grplist.append( grp.name )
            grplist.sort()
            if len( grplist ) > 0:
                parent = "<b>%s</b>" % cat.name
                root = self.model.append( None, [parent, "", 0] )
                for grp in grplist:
                    g = comps.return_group( grp )
                    if g.installed:
                        state = 1
                    else:
                        state = 0
                    self.model.append( root, [g.name, g.description, state] )
                    
    def populate_view_yum24( self, groupInfo ):
        """ Populate a Notebook page with data """
        self.model.clear()
        parent = "<b>%s</b>" % _("All Groups")
        root = self.model.append( None, [parent, "", False] )
        for grp in groupInfo.group_by_id.keys():
            g = groupInfo.group_by_id[grp]
            if g.user_visible:
                installed = groupInfo.isGroupInstalled(grp)
                if installed:
                    state = 1
                else:
                    state = 0
                self.model.append( root, [g.name, g.description, state] )

class pkgView:
    """ 
        This class handles the Install/Update/Remove TreeViews.
        
        It contains the following model:
        
        TreeView <- TreeModelFilter <- TreeModelSort <- Liststore
        
         - Liststore contains yumexPackage objects.
         - TreeModelFilter
           This model is used to split up the yumexPackage objects
           attributes into TreeView columns and to control what packages 
           there is shown, depending on the visible state of the yumexPackage
           objects.
         - TreeModelSort is used to control the sorting of Columns.
      
        it contains all the event handles for package selction, selection
        column click and recent column click and methods for reading/writing
        selected packages.
        
    """
    def __init__( self,  progress):
        #self.view = widget
        self.headers = [_( "Package" ), _( "Ver" ), _( "Summary" ), _( "Repo" ), _( "Architecture" ), _( "Size" )]
        self.filterTypes = [gobject.TYPE_BOOLEAN, str, str, str, str, str, str, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN,gobject.TYPE_BOOLEAN]
        # List of attributes in pkg object to use in TreeView Columns
        self.filterAttrs = ['selected', 'name', 'ver', 'summaryFirst', 'repoid', 'arch', 'sizeFmt', 'recent', 'visible','queued']
        self.packageList = yumexPackageList()
        self.model, self.liststore, self.filter = self.setup_view()
        self.SelectState = False
        self.RecentState = 0
        self.progress = progress
    
    def on_toggled( self, widget, path ):
        """ Package selection handler """
        row = self.get_child_path_row( path )
        pkg = row[0]
        newstate = not pkg.selected
        pkg.set_select( newstate )
        
    def get_pkg( self,model, iter ):
        """ Package selection handler """

        path = model.get_string_from_iter(iter)
        row = self.get_child_path_row( path )
        pkg = row[0]
        
        return pkg
        

    def get_child_path_row( self, path ):
        """ Get Row in ListStore from SortModel Path """  
        # Get FilterModel path from SortModel path
        filterpath = self.model.convert_path_to_child_path( path )
        # Get ListStore Path from Filter Model Path
        storepath = self.filter.convert_path_to_child_path( filterpath )
        # Get Row From liststore
        row = self.liststore[storepath]
        return row
        
    def sort_name_column( self, model, iter1, iter2 ):
        """ 
        Packagename sorting function 
        sorts by name-ver.arch
        """
        name = model.get_value( iter1, 1 )
        ver = model.get_value( iter1, 2 )
        arch = model.get_value( iter1, 5 )
        pkg_name1 = "%s-%s.%s" % ( name, ver, arch )
        name = model.get_value( iter2, 1 )
        ver = model.get_value( iter2, 2 )
        arch = model.get_value( iter2, 5 )
        pkg_name2 = "%s-%s.%s" % ( name, ver, arch )
        if pkg_name1 < pkg_name2:
            return -1
        elif pkg_name1 > pkg_name2:
            return 1
        else:
            return 0
        
    def sort_repo_column( self, model, iter1, iter2 ):
        """ 
        repo sorting function 
        sorts by repo.name-ver.arch
        """
        name = model.get_value( iter1, 1 )
        ver = model.get_value( iter1, 2 )
        repo = model.get_value( iter1, 4 )
        arch = model.get_value( iter1, 5 )
        elem1 = "%s.%s-%s.%s" % ( repo, name, ver, arch )
        name = model.get_value( iter2, 1 )
        ver = model.get_value( iter2, 2 )
        repo = model.get_value( iter2, 4 )
        arch = model.get_value( iter2, 5 )
        elem2 = "%s.%s-%s.%s" % ( repo, name, ver, arch )
        if elem1 < elem2:
            return -1
        elif elem1 < elem2:
            return 1
        else:
            return 0
            
            
            
        
    def sort_size_column( self, model, iter1, iter2 ):
        """ 
        size sorting function 
        """
        
        size1 = float( self.get_pkg(model,iter1).size )
        size2 = float( self.get_pkg(model,iter2).size )
        if size1 < size2:
            return -1
        elif size1 > size2:
            return 1
        else:
            return 0

    def on_select_column_click( self, col ): 
        """ 
        Select Column Click Handler
        set/reset : Show only selected packages filter.        
        """
        self.SelectState = not self.SelectState
        if self.packageList != None:
            self.packageList.apply_filter( None, self.RecentState, self.SelectState )
            data = self.packageList.get_active_list()
            self.update_view()

    def on_recent_column_click( self, col ): 
        """
        Recent Column Click Handler
        Set/Reset : Show only recent packages filter.
        """
        if self.RecentState < 2:
            self.RecentState += 1
        else:
            self.RecentState = 0
        self.packageList.apply_filter( None, self.RecentState, self.SelectState )
        self.update_view()
                
    def filter_modify( self, model, iter, col ):
        """
        TreeModelFilter modify medthod.
        Split up the yumexPackage object attributes into column values
        to show in the TreeView.
        """
        # Convert the filter iter to the corresponding child model iter.
        child_model_iter = model.convert_iter_to_child_iter( iter )
        #child_model = model.get_model()
        obj = self.liststore.get_value( child_model_iter, 0 )
        if obj != None:
            val = getattr( obj, self.filterAttrs[col] )
            return val
        else:
            print child_model_iter, col
            return None
             
    def filter_visible( self, model, iter ):
        """
        TreeModelFilter filter medthod.
        Control if a package is shown, based on the visible
        attribute in the yumexPackage object.
        """
        pkg = model.get_value( iter, 0 )
        if pkg != None:
            return pkg.visible
        else:
            return False
    
    
    def setup_view( self ):
        """
        Setup Models, TreeView Columns and connect event handlers.
        """
        liststore = self.packageList.list
        modelfilter = liststore.filter_new()
        modelfilter.set_modify_func( self.filterTypes, self.filter_modify )
        modelfilter.set_visible_func( self.filter_visible )
        
        model = gtk.TreeModelSort( modelfilter )   
        
        model.set_sort_column_id( 1, gtk.SORT_ASCENDING )
        model.set_sort_func( 1, self.sort_name_column )
        model.set_sort_func( 4, self.sort_repo_column )
        model.set_sort_func( 6, self.sort_size_column )

#        self.view.set_model( model )
#        
#        # Setup selection column
#        cell1 = gtk.CellRendererToggle()    # Selection
#        cell1.set_property( 'activatable', True )
#        column1 = gtk.TreeViewColumn( "    ", cell1 )
#        column1.add_attribute( cell1, "active", 0 )
#        column1.set_resizable( True )
#        column1.set_sort_column_id( -1 )            
#        self.view.append_column( column1 )
#        cell1.connect( "toggled", self.on_toggled, ( model, liststore ) )            
#        column1.set_clickable( True )
#        column1.connect( "clicked", self.on_select_column_click ) 
#
#        # Setup resent column
#        cell2 = gtk.CellRendererPixbuf()    # new
#        cell2.set_property( 'stock-id', gtk.STOCK_ADD )
#        column2 = gtk.TreeViewColumn( "    ", cell2 )
#        column2.set_cell_data_func( cell2, self.new_pixbuf )
#        column2.set_resizable( True )
#        column2.set_sort_column_id( -1 )            
#        self.view.append_column( column2 )
#        column2.set_clickable( True )
#        column2.connect( "clicked", self.on_recent_column_click ) 
#        
#        for i in [1, 2, 3, 4, 5, 6]:       
#            self.create_text_column( self.headers[i-1], self.view, i )
#        model.set_sort_column_id( 1, gtk.SORT_ASCENDING )
#        model.set_sort_func( 1, self.sort_name_column )
#        model.set_sort_func( 4, self.sort_repo_column )
#        model.set_sort_func( 6, self.sort_size_column )
#        self.view.set_search_column( 1 )
#        self.view.set_reorderable( False )
        return model, liststore, modelfilter
   
    def create_text_column( self, hdr, view, colno ):
        """ 
        Create a TreeViewColumn with text and set
        the sorting properties and add it to the view
        """
        cell = gtk.CellRendererText()    # Size Column
        column = gtk.TreeViewColumn( hdr, cell, text=colno )
        column.set_resizable( True )
        if not colno in  [2, 5]:
            column.set_sort_indicator( True )
            column.set_sort_column_id( colno )
            column.set_sort_order( gtk.SORT_ASCENDING ) 
        else:
            column.set_sort_column_id( -1 )
        view.append_column( column )        
         
#    def new_pixbuf( self, column, cell, model, iter ):
#        """ 
#        Cell Data function for recent Column, shows pixmap
#        if recent Value is True.
#        """
#        isNew = model.get_value( iter, 7 )
#        cell.set_property( 'visible', isNew )

       
    def clear( self ):
        #self.liststore.clear()
        self.packageList.clean()

    def add( self, pkg, avail, recentlimit ):    
        ypkg = self.packageList.add( pkg, avail, recentlimit )
        #self.liststore.append( [ypkg] )         
        
    def lenght( self ):  
        return self.packageList.get_lenght()

#    def reconnect_view(self):
#        gtk.threads_enter() # Make it thread safe        
#        """Reconnect to view model """
#        self.model.set_sort_func(1,self.sort_name_column)
#        self.view.set_model( self.model )
#        self.view.set_search_column(1)
#        self.view.set_reorderable(False)        
#        self.view.thaw_child_notify()
#        gtk.threads_leave()        
#
#    def disconnect_view(self):
#        """ 
#        Disconect the TreeView from liststore and disables the sort function
#        and stops childs notification.
#        This is done, because it make it faster to insert data into liststore.
#        """
#        gtk.threads_enter() # Make it thread safe        
#        self.view.set_model(None) # disconnect view from model (much faster)
#        no_sort_func = lambda *args: 0
#        self.model.set_sort_func(1,no_sort_func)            
#        self.view.freeze_child_notify()
#        gtk.threads_leave()        

    def update_view( self ):
        """ Updates view by making the TreeModelFilter do a refilter """
        self.filter.refilter()

    def apply_filter( self, flt ):
        """ Apply filter to current packageList and update the view """
        self.packageList.apply_filter( flt, self.RecentState, self.SelectState )
        self.update_view()
            
    def get_selected( self, package=True ):
        """ Get selected packages in current packageList """
        return self.packageList.get_selected()

    def get_lenght( self ):
        """ Get number of packages in current packageList """
        return self.packageList.get_lenght()
            
    def deselect_all( self ):
        """ Deselect all packages in current packageList """
        self.packageList.deselect_all()
        self.update_view()

    def select_all( self ):
        """ Select all packages in current packageList"""
        self.packageList.select_all()              
        self.update_view()

    def select_by_keys( self, keys, isPkgs=True ):
        """ 
        Select all packages, included in a list of
        name.arch-ver
        """
        self.packageList.select_by_fullname( keys )
        self.update_view()
            
          
  
class repoView:
    """ 
    This class controls the repo TreeView
    """
    def __init__( self, widget, headers=None ):
        self.view = widget
        self.headers = headers
        self.model, self.liststore = self.setup_view()
    
    
    def on_toggled( self, widget, path, data ):
        """ Repo select/unselect handler """
        row = self.get_child_path_row( path )
        newstate = not row[0]
        row[0] = newstate

    def get_child_path_row( self, path ):
        """ Get Row in ListStore from SortModel Path """  
        # Get ListStore Path from SortModel Path
        storepath = self.model.convert_path_to_child_path( path )
        # Get Row From liststore
        row = self.liststore[storepath]
        return row
        
                
    def setup_view( self ):
        """ Create models and columns for the Repo TextView  """
        liststore = gtk.ListStore( 'gboolean', gobject.TYPE_STRING, gobject.TYPE_STRING )
        model = gtk.TreeModelSort( liststore )   
        model.set_sort_func( 2, self.sort_file_column )
        self.view.set_model( model )
        # Setup Selection Column
        cell1 = gtk.CellRendererToggle()    # Selection
        cell1.set_property( 'activatable', True )
        column1 = gtk.TreeViewColumn( "    ", cell1 )
        column1.add_attribute( cell1, "active", 0 )
        column1.set_resizable( True )
        column1.set_sort_column_id( -1 )            
        self.view.append_column( column1 )
        cell1.connect( "toggled", self.on_toggled, ( model, liststore ) )            
        # Setup reponame & repofile column's
        for i in [1, 2]:     
            self.create_text_column( self.headers[i-1], self.view, i )
        self.view.set_search_column( 1 )
        self.view.set_reorderable( False )
        return model, liststore
    
    def create_text_column( self, hdr, view, colno ):
        cell = gtk.CellRendererText()    # Size Column
        column = gtk.TreeViewColumn( hdr, cell, text=colno )
        column.set_resizable( True )
        column.set_sort_indicator( True )
        column.set_sort_column_id( colno )
        column.set_sort_order( gtk.SORT_ASCENDING ) 
        view.append_column( column )        

    def sort_file_column( self, model, iter1, iter2 ):
        """ 
        repo file sorting function 
        """
        file = model.get_value( iter1, 2 )
        id  = model.get_value( iter1, 1 )
        elem1 = "%s%s" % ( file,id )
        file = model.get_value( iter2, 2 )
        id  = model.get_value( iter2, 1 )
        elem2 = "%s%s" % ( file,id )
        if elem1 < elem2:
            return -1
        elif elem1 > elem2:
            return 1
        else:
            return 0


    def populate_view( self, data ):
        """ Populate a repo liststore with data """
        self.liststore.clear()
        for state, repo, file in data:
            iterator = self.liststore.append()           
            self.liststore.set_value( iterator, 2, file )
            self.liststore.set_value( iterator, 1, repo )
            self.liststore.set_value( iterator, 0, state )
            
    def get_selected( self ):
        selected = []
        for elem in self.liststore:
            state = elem[0]
            name = elem[1]
            if state:
                selected.append( name )
        return selected
        
    def get_notselected( self ):
        notselected = []
        for elem in self.liststore:
            state = elem[0]
            name = elem[1]
            if not state:
                notselected.append( name )
        return notselected
            
    def deselect_all( self ):
        iterator = self.liststore.get_iter_first()
        while iterator != None:    
            self.liststore.set_value( iterator, 0, False )
            iterator = self.liststore.iter_next( iterator )

    def select_all( self ):
        iterator = self.liststore.get_iter_first()
        while iterator != None:    
            self.liststore.set_value( iterator, 0, True )
            iterator = self.liststore.iter_next( iterator )
            

    def select_by_keys( self, keys, isPkgs=True ):
        liststore = self.liststore
        iterator = liststore.get_iter_first()
        while iterator != None:    
            if isPkgs:
                pkg = self.get_fullpkg( iterator )
            else:
                pkg = liststore.get_value( iterator, 1 )
            if pkg.lower() in keys:
                liststore.set_value( iterator, 0, True )
            else:
                liststore.set_value( iterator, 0, False )
            iterator = liststore.iter_next( iterator )

class yumexConf( BaseConfig ):
    """ Yum Extender Config Setting"""
    autorefresh = BoolOption( True )
    filelists = BoolOption( False )
    recentdays = IntOption( 14 )
    autocleanup = BoolOption( False )
    mirrordetection = SelectionOption( 'default', ( 'default', 'fastest', 'best' ) )
    debug = BoolOption( False )
    noplugins = BoolOption( False )
    usecache = BoolOption( False )
    proxy = Option()
    exclude = ListOption()
    fullobsoletion = BoolOption( False )
    processsafemode = BoolOption( False )
    
class mirrorCacheConf( BaseConfig ):
    urls = ListOption()
    sortkeys = ListOption()
    detection = SelectionOption( 'default', ( 'default', 'fastest', 'best' ) )
    lastscan = FloatOption()
    
class mirrorCache:
    def __init__( self, parser, file ):
        self.conf = {}
        self.parser = parser
        self.file = file

    def add( self, name ):
        if not self.conf.has_key( name ):
            self.conf[name] = mirrorCacheConf()
            if not self.parser.has_section( name ):
                self.parser.add_section( name )
            self.conf[name].populate( self.parser, name )
 
            
    def get( self, name ):
        if self.conf.has_key( name ):
            return self.conf[name]
        else:
            return None
            
    def get_names( self ):
        return self.conf.keys()
        
    def has_repo( self, name ):    
        return self.conf.has_key( name )
        
    def load( self ):
        self.parser.read( self.file )
        for sec in self.parser.sections():
            if self.conf.has_key( sec ):
                self.conf[sec].populate( self.parser, sec )
            else:
                self.add( sec )
                self.conf[sec].populate( self.parser, sec )
        
    def save( self ):
        fn = file( self.file, 'wt' )
        self.parser.write( fn )
        fn.close()

    def add_repo( repo, keys, detection, lastscan ):
        r = repo.id
        if not self.conf.has_key( r ):
            self.add( r )
        self.conf[r].repo = repo.id
        self.conf[r].urls = repo.urls
        self.conf[r].keys = keys
        self.conf[r].detection = detection
        self.conf[r].lastscan = lastscan
   
            
        
def read_yumex_conf( configfile='/etc/yumex.conf', sec='yumex' ):
    conf = yumexConf()
    parser = IncludingConfigParser()
    parser.read( configfile )
    conf.populate( parser, sec )
    return conf
    

#class Settings:
#    """ This class is used to read and write configuration from
#        /etc/yumex.conf """
#        
#    def __init__( self, cfgfile='/etc/yumex.conf', sec='yumex' ):
#        self.conf_file = cfgfile
#        self.section = sec
#        self.conf = ConfigParser.ConfigParser()
#        self.conf.read( self.conf_file )             
#        if not self.conf.has_section( self.section ):
#            self.conf.add_section( self.section )
#            
#
#    def get_option( self, option ):
#        if self.conf.has_option( self.section, option ):
#            return self.conf.get( self.section, option )
#        else:
#            return None
#        
#
#    def is_enabled( self, option ):
#        if self.conf.has_option( self.section, option ):
#            if self.conf.get( self.section, option ) == "1":
#                return True
#            else:
#                return False
#        else:
#            return False
#
#    def load( self ):
#        self.conf.read( self.conf_file )             
#        
#    def get_value( self, option ):
#        if self.conf.has_option( self.section, option ):
#            return self.conf.get( self.section, option )
#        else:
#            return None
#
#        
#    def set_value( self, option, value="" ):
#        if value != "":
#            self.conf.set( self.section, option, value )
#        else:
#            if self.conf.has_option( self.section, option ):
#                self.conf.remove_option( self.section, option )
#    
#    def set_boolean( self, option, state ):
#        if state:
#            self.conf.set( self.section, option, "1" )
#        else:
#            self.conf.set( self.section, option, "0" )
#
#    def get_boolean( self, option ):
#        if self.conf.has_option( self.section, option ):
#            if self.conf.get( self.section, option ) == "1":
#                return True
#            else:
#                return False
#        else:
#            return False
#        
#    def save( self ):
#        f=open( self.conf_file, "w" )
#        self.conf.write( f )
#        f.close()        
#        
#    def remove_section_elem( self ):
#        if self.conf.has_section( self.section ):
#            items = self.conf.options( self.section )
#            for i in items:
#                self.conf.remove_option( self.section, i )
#                
#            
#    def add_section( self ):
#        if not self.conf.has_section( self.section ):
#            self.conf.add_section( self.section )
        


class Profile:
    """ Class for handling repo selection Profile """
    def __init__( self ):
        self.profiles = ConfigParser.ConfigParser()
        self.filename = '/etc/yumex.profiles.conf'
        self.profiles.read( self.filename )
        self.active=self.profiles.get( "main", "LastProfile" )    
        self.proDict = {}    
        self.load()        

    
    def save( self ):
        f=open( self.filename, "w" )
        self.profiles.write( f )
        f.close()
        
    def load( self ):
        self.profiles.read( self.filename )
        self.active=self.profiles.get( "main", "LastProfile" )        
        for p in self.profiles.sections():
            if p != 'main':
                opts = self.profiles.options( p )
                self.proDict[p] = opts
        #print self.proDict

    def get_profile( self, name ):
        if name in self.proDict.keys():
            return self.proDict[name]
        else:
            return False

    def set_profile( self, name, repos ):
        if self.profiles.has_section( name ):
            self.profiles.remove_section( name )    
            self.add_profile( name, repos )                
            return True
        else:
            return False
        

    def get_list( self ):
        return self.proDict.keys()

    def get_active( self ):
        return self.active

    def set_active( self, name ):
        if self.profiles.has_section( name ) or name == "yum-enabled":
            self.active = name
            self.profiles.set( "main", "LastProfile", name )        
            self.save()

    def add_profile( self, name, repos ):
        if not self.profiles.has_section( name ):
            self.profiles.add_section( name )
            for r in repos:
                self.profiles.set( name, r, "1" )
            self.save()
            self.load()
            return True
        else:
            return False
            

class Repository:
    """ Class for handling .repo files """
    def __init__( self ):
        self.baseRepos = ['updates-released', 'base', 'development', 'updates-testing']
        self.cfg = ConfigParser.ConfigParser()
        self.update()
        #self.list_all()
    
    def list_all( self ):                                    # List all repos.
        for r in self.repos:
            self.set_active( r )
            print "[%s]" % r
            opt = self.list_options()
            for o in opt:
                if o in ["enabled", "gpgcheck"]:
                    bu = self.get_flag( o )
                    print o, " : ", bu
                    print "%s=%s" % ( o, self.get_option( o ) )
                else:
                    print "%s=%s" % ( o, self.get_option( o ) )
            
    def list_active( self ):                            # List active repo
        print "[%s]" % self.activeRepo
        print "%s=%s" % ( "name", self.get_option( "name" ) )
        if self.isBaseUrl():
            print "%s=%s" % ( "baseurl", self.get_option( "baseurl" ) )
        else:
            print "%s=%s" % ( "mirrorlist", self.get_option( "mirrorlist" ) )
        opt = self.list_options()
        for o in opt:                                            
            if o in ['name', 'baseurl', 'mirrorlist']:    
                pass
            else:
                value = self.get_option( o )                    
                print "%s=%s" % ( o, value )
        
                                            
    def load_repos( self ):                    # Load repo information from repofiles
        files = self.fileDict.keys()        # Load repo info for files in file dict.
        for f in files:
            self.cfg.read( f )
        if self.cfg.has_section( 'main' ):    # Remove yum.conf [main] if exist
            self.yum_conf_main = self.cfg.items( 'main' )
            self.cfg.remove_section( 'main' )
        
    def build_section_dict( self ):            # build repo section, repo file directory
        tcfg = ConfigParser.ConfigParser()
        r = '/etc/yum.conf'
        tcfg.read( r )
        for s in tcfg.sections():            # Parse sections
            #print "    Section : "+s
            if s != "main":
                self.repoDict[s] = r
                if r in self.fileDict.keys():
                    self.fileDict[r] =  self.fileDict[r]+[s]
                else:
                    self.fileDict[r] = [s]
        if len( tcfg.sections() ) > 1: 
            ycRepos = self.fileDict[r]
        else:
            ycRepos =[]
        #print ycRepos
        for r in self.repofiles:                # Parse .repo files
            tcfg = ConfigParser.ConfigParser()
            tcfg.read( r )
            #print "File : "+r    
            for s in tcfg.sections():            # Parse sections
                #print "    Section : "+s
                if s in ycRepos:                    # Check if repos allready definded in yum.conf
                    shutil.copy( r, r+".yumex" )        # Rename .repo file to .repo.yumex
                    os.remove( r )                        # Remove file
                else:
                    self.repoDict[s] = r
                    if r in self.fileDict.keys():
                        self.fileDict[r] = self.fileDict[r] + [s]
                    else:
                        self.fileDict[r] = [s]
        #print self.repoDict
        #print self.fileDict
        
    def update( self ):                            # Updates class member, reloads repos
        
        self.repofiles = glob.glob( '/etc/yum.repos.d/*.repo' )
        #self.repofiles = glob.glob('repos/*.repo')
        self.repofiles.sort()
        self.repoDict =  {}
        self.fileDict = {}
        self.yum_conf_main = []
        self.build_section_dict()
        self.load_repos()
        self.repos = self.read_names()
        self.filename = ""
        self.activeRepo = ""
        
    def isBaseUrl( self ): # If repo has baseurl=
        return self.cfg.has_option( self.activeRepo, "baseurl" )
            
    def read_names( self ):    # return all repo titles.
        return self.cfg.sections()
    
    def set_active( self, repo ):    # set active repo
        self.activeRepo = repo
    
    def get_flag( self, option, default=False ):    # read flag from active repo
        try:
            return self.cfg.getboolean( self.activeRepo, option )
        except ( ConfigParser.NoSectionError, ConfigParser.NoOptionError ), e:                
            return default

    def list_options( self ):    # returns all options from active repo
        return self.cfg.options( self.activeRepo )
    
    def get_option( self, option, default=None ):    # read option value from active repo
        try:
            return self.cfg.get( self.activeRepo, option )
        except ( ConfigParser.NoSectionError, ConfigParser.NoOptionError ), e:
            return default

    def set_flag( self, option, state ):                # Set flag in active Repo
        if state:
            return self.cfg.set( self.activeRepo, option, "1" )
        else:
            return self.cfg.set( self.activeRepo, option, "0" )
        
    
    def set_option( self, option, value ):                # Set option in active repo
        return self.cfg.set( self.activeRepo, option, value )

    def remove_option( self, option ):                    # remove option in active repo
        return self.cfg.remove_option( self.activeRepo, option )
    
    def set_filename( self, key ):                        # set filename from title lookup
        self.filename = self.repoDict[key]
        #print self.filename
        
    def get_repo_filename( self, key ):
        return self.repoDict[key]     
        
    def get_active_repos( self ):           
        repos = self.read_names()
        enabled_repos = []
        for r in repos:        
            self.set_active( r )
            enabled=self.get_flag( "enabled",True )
            if enabled:
                enabled_repos.append( r )
        return enabled_repos

    def get_names_sorted( self ):    # Returns sort list of repos (base repos first)
        blist = []
        olist = []
        repos = self.read_names()
        for r in repos:        
            if r in self.baseRepos:
                blist.append( r )
        blist.sort()
        for r in repos:
            if r in self.baseRepos:
                pass
            else:
                olist.append( r )
        olist.sort()    
        blist.extend( olist )    # Append otherlist in end of baselist
        return blist
        
    def save( self ):        # Save active repo to .repo file.
        sec = self.activeRepo
        self.filename = self.repoDict[sec]
        shutil.copy( self.filename, self.filename+".bak" )    # Create Backup
        self.save_repo_file()
        self.update()                                # Reload repos
        self.activeRepo = sec    

    def list_exclude( self, list1, list2 ):
        data = []
        for el in list1:
            if el not in list2:
                data.append( el )
        return data


    def delete_repo( self, name ):
        if name in self.repos:
            f = self.repoDict[name]             # File were name is stored
            repos = self.fileDict[f]            # All repos in this file
            self.filename = f
            if len( repos ) == 1:                # is name the only repo in file 
                os.remove( f )                   # Remove file
                self.cfg.remove_section( name ) 
                del self.fileDict[f]             # remove repo file from fileDirec     
            else:
                self.cfg.remove_section( name )                        
                self.fileDict[f] = self.list_exclude( repos, [name] ) # repos - name.
                self.save_repo_file()                
            self.update()  
             
                        
    def create_new( self, title, name=None, urltype="baseurl", url="", enable="0", gpg="0", gpgkey=None ):
        """ Create new Repository """
        if title in self.repos:
            sys.stderr.write( "Repository with name %s allready exist..\n" % title )
            return False
        self.filename = "/etc/yum.repos.d/"+title+".repo"
        self.cfg.add_section( title )
        if name == None:
            name = title
        self.cfg.set( title, "name", name )
        if urltype == "baseurl":
            self.cfg.set( title, "baseurl", url )
        else:
            self.cfg.set( title, "mirrorlist", url )
        self.cfg.set( title, "enabled", enable )
        self.cfg.set( title, "gpgcheck", gpg )
        if gpgkey != None:
            self.cfg.set( title, "gpgkey", gpgkey )
        self.activeRepo = title 
        self.fileDict[self.filename] = [title]       
        self.save_repo_file()
        self.update()                                # Reload repos
        self.activeRepo = title                
        #self.list_active()
        return True

    def write_repo_to_file( self, f ):
        f.write( "#\n" ) 
        f.write( "# Repository : %s \n" % self.activeRepo )
        f.write( "#\n" ) 
        f.write( "[%s]\n" % self.activeRepo )
        f.write( "%s=%s\n" % ( "name", self.get_option( "name" ) ) )
        if self.isBaseUrl():
            urls = self.get_option( "baseurl" ).split( '\n' )
            #print urls
            first = True
            for url in urls:
                if first:
                    f.write( "%s=%s\n" % ( "baseurl", url ) )
                    first = False
                elif url.strip() != '':
                    f.write( "          %s\n" % ( url ) )                  
        else:
            f.write( "%s=%s\n" % ( "mirrorlist", self.get_option( "mirrorlist" ) ) )
        opt = self.list_options()
        for o in opt:                                            
            if o in ['name', 'baseurl', 'mirrorlist']:    
                pass
            else:
                value = self.get_option( o )                    
                f.write( "%s=%s\n" % ( o, value ) )
        f.write( "\n" ) 


    def save_repo_file( self ):
        #print "Saving to : "+self.filename  
        sections = self.fileDict[self.filename]
        #print sections        
        f=open( self.filename, "w" )
        if self.filename == '/etc/yum.conf':
            self.write_yum_main( f )
        today = datetime.date.today()
        f.write( "# Repositories updated : %s\n\n" % today )
        for s in sections:
            self.activeRepo = s
            self.write_repo_to_file( f )
        f.close()
    
    def write_yum_main( self, f ):
        f.write( "#\n" ) 
        f.write( "# Yum main Configuration\n" )
        f.write( "#\n" ) 
        f.write( "[main]\n" )
        for key, value in self.yum_conf_main:
            f.write( "%s=%s\n" % ( key, value ) )
        f.write( "\n# PUT YOUR REPOS HERE OR IN separate files named file.repo\n" )
        f.write( "# in /etc/yum.repos.d\n\n" )

class BackgroundRunner( Thread ):
    """ This Class runs a function as a background Tread"""
    def __init__ ( self, runfunc, args ):
        Thread.__init__( self )
        self.runfunc = runfunc
        self.args = args
        self.rc = None

    def run ( self ):
        #print "Running as Thread  : ", self.runfunc
        #print "--> Parameters     : ", self.args
        self.rc = self.runfunc( *self.args )
        #print "Running Completted : "



    
# Misc Functions        
        
def format_number( number, SI=0, space=' ' ):
    """Turn numbers into human-readable metric-like numbers"""
    symbols = ['', # (none)
                'k', # kilo
                'M', # mega
                'G', # giga
                'T', # tera
                'P', # peta
                'E', # exa
                'Z', # zetta
                'Y'] # yotta

    if SI: step = 1000.0
    else: step = 1024.0

    thresh = 999
    depth = 0

    # we want numbers between 
    while number > thresh:
        depth  = depth + 1
        number = number / step

    # just in case someone needs more than 1000 yottabytes!
    diff = depth - len( symbols ) + 1
    if diff > 0:
        depth = depth - diff
        number = number * thresh**depth

    if type( number ) == type( 1 ) or type( number ) == type( 1L ):
        format = '%i%s%s'
    elif number < 9.95:
        # must use 9.95 for proper sizing.  For example, 9.99 will be
        # rounded to 10.0 with the .1f format string (which is too long)
        format = '%.1f%s%s'
    else:
        format = '%.0f%s%s'

    return( format % ( number, space, symbols[depth] ) )

# Routine by Micah D. Cochran
# Submitted on 26 Aug 2005
# This routine is allowed to be put under any license Open Source (GPL, BSD, LGPL, etc.) License 
# or any Propriety License. Effectively this routine is in public domain. Please attribute where appropriate.

def strip_ml_tags( in_text ):
    """Description: Removes all HTML/XML-like tags from the input text.
    Inputs: s --> string of text
    Outputs: text string without the tags
    
    # doctest unit testing framework

    >>> test_text = "Keep this Text <remove><me /> KEEP </remove> 123"
    >>> strip_ml_tags(test_text)
    'Keep this Text  KEEP  123'
    """
    # convert in_text to a mutable object (e.g. list)
    s_list = list( in_text )
    i, j = 0, 0
    
    while i < len( s_list ):
        # iterate until a left-angle bracket is found
        if s_list[i] == '<':
            while s_list[i] != '>':
                # pop everything from the the left-angle bracket until the right-angle bracket
                s_list.pop( i )
                
            # pops the right-angle bracket, too
            s_list.pop( i )
        else:
            i=i+1
            
    # convert the list back into text
    join_char=''
    return join_char.join( s_list )        